<?php

// To run this proxy as a stand-alone script, edit the options below to the appropriate values ( as-is they use the RePress Wordpress options )

/** -- options -- **/

$rewritebase = get_option('repress_rewritebase');
$remoteservers = json_decode( get_option('repress_remoteservers'), TRUE );
$agentstring = "RePress plugin/" . plugin_get_version() . " (PHP " . phpversion() . " streamer) - please read https://all4xs.net";
if (get_option('repress_push_post') == 'yes') {
	$doPosts = true;
} else {
	$doPosts = false;
}
if (get_option('repress_push_cookie') == 'yes') {
	$doCookies = true;
} else {
	$doCookies = false;
}
$GLOBALS['proxydebug'] = false;		// debugging?
$proxyhelp = true;			// use more sophisticated phproxyimproved html rewriting
error_reporting(E_NONE);

/** -- end of options -- **/

// Proxy code begins here.

// dependencies

if ($proxyhelp) {
	// proxyhelp requirement
	require_once('proxyhelp/proxify_html_resource.php');
}

/* Process request */

logline("Proxy intercepted request: " . $_SERVER['REQUEST_URI']);

$hit = false; $extra = ''; $remoteserver = ''; $abbreviation = ''; $text = false;

// is rewrite base contained in the request url? then start looking for a hit

if (strstr('/' . $_SERVER['REQUEST_URI'], $rewritebase . '/') !== FALSE) {

	logline("/$rewritebase/ in our url. Seems promising.");

	foreach ($remoteservers as $hostname) {
		// future work: support hash matching in URLs instead of hostnames.
		logline("$hostname -> match checking");

		// match the subdomains first

		$rmatch = preg_replace("/\//", "", $rewritebase);
		if (preg_match("/$rmatch\/(.*?)\.".$hostname."[\/]?$/", $_SERVER['REQUEST_URI'], $matches)) {
                        logline("matched on subdomain with no extra");
                        $hit = true; $remoteserver = $matches[1] . '.' . $hostname; $abbreviation = $matches[1] . '.' . $hostname; $extra = ''; break;
                } elseif (preg_match("/$rmatch\/(.*?)\.$hostname\/(.*)$/", $_SERVER['REQUEST_URI'], $matches)) {
                        logline("matched on subdomain with extra");
                        $hit = true; $remoteserver = $matches[1] . '.' . $hostname; $abbreviation = $matches[1] . '.' . $hostname; $extra = $matches[2]; break;
                }  elseif (preg_match("/$rmatch\/" . $hostname . "[\/]?$/", $_SERVER['REQUEST_URI'])) {
                        logline("matched on main domain with no extra");
                        $hit = true; $remoteserver = $hostname; $abbreviation = $hostname; $extra = ''; break;
                } elseif (preg_match("/$rmatch\/" . $hostname . "\/(.*)$/", $_SERVER['REQUEST_URI'], $matches)) {
                        logline("matched on main domain with extra");
                        $hit = true; $remoteserver = $hostname; $abbreviation = $hostname; $extra = $matches[1]; break;
                } else {
                        logline("no rules matched");
                }
	}

}

if ($hit) {

	// try to protect against XSS attacks on Wordpress sessions if running as plugin
	if (function_exists('repress_logout_wordpress_user')) {
		repress_logout_wordpress_user();
	}

	// determine net link by stripping the ?GET parameters
	$fopenLink = preg_replace("/\?.*$/", "", "http://$remoteserver/$extra");

	logline("Interpreted! hit as follows:");
	logline("Remoteserver = $remoteserver, abbreviation = $abbreviation, extra = $extra and net open link is http://$remoteserver/$extra");

	// do we do a POST?

	if ($doPosts && is_array($_POST) && count($_POST) > 0) {
		$requestType = 'POST';
	} else {
		$requestType = 'GET';
	}

	// do we do cookies?

	$cookieHeaders = '';

	if ($doCookies && is_array($_COOKIE) && count($_COOKIE) > 0) {
		foreach ($_COOKIE as $name => $val) {
			$encodeName = rawurlencode($name);
			$encodeVal = rawurlencode($val);
			$cookieHeaders .= "Cookie: $encodeName=$encodeVal\r\n";
		}
	}

	/* Establish streaming connection to the remote server */

	if ($requestType == 'GET') {
		$streamOptions = array(
					  'http'=>array(
        	           			  'method'=>"GET",
						  'host'=>$remoteserver,
						  'user_agent'=>$agentstring,
						  'header'=>$cookieHeaders
			         	   )
				      );
	} else {
		$postData = http_build_query($_POST);

		$streamOptions = array(
					  'http'=>array(
       	        	    			  'method'=>"POST",
						  'host'=>$remoteserver,
						  'user_agent'=>$agentstring,
						  'header'=>"Content-type: application/x-www-form-urlencoded\r\n" .
						            "Content-Length: " . strlen($postData) . "\r\n" .
							    $cookieHeaders,
						  'content'=>$postData
			         	   )
				      );
	}

	$context = stream_context_create($streamOptions);
	logline("Open to location " . "http://$remoteserver/$extra");
	$handle = fopen("http://$remoteserver/$extra", "rb", false, $context);
	if ($handle == FALSE) {
		if (isset($GLOBALS['proxydebug']) && $GLOBALS['proxydebug']) {
			echo("Proxy error to: http://$remoteserver/$extra");
		}
		exit;
	}

	/* Start working on headers */

	$meta = stream_get_meta_data($handle);
	$headerData = $meta['wrapper_data'];

	if (preg_match("/HTTP.... 30./", $headerData[0])) $redirect = true;
	else {
		$redirect = false;
	}

	$i = 0;
	foreach ($headerData as $line) {
		$i++;

		if ($i > 1 && preg_match("/^HTTP\/1../", $line)) {
			// no multiple HTTP headers will be replied
			exit;
		}

		// make 302 redirects work
		if ($redirect && !preg_match("#$rewritebase#", $line)) {

			// a redirect to root is a redirect to main page here
			$line = preg_replace("#^Location: /#", "Location: $rewritebase/$abbreviation/", $line);

			// we localize redirects to all known hosts, and subdomains of those hosts
			foreach ($remoteservers as $knownhost) {
				$knownhostAbbr = $knownhost;			// future work: support hashing

				// an absolute redirect to a main page
				$line = preg_replace("#^Location: http://$knownhost#", "Location: $rewritebase/$knownhostAbbr", $line);
				$line = preg_replace("#^Location: http://$knownhost/#", "Location: $rewritebase/$knownhostAbbr", $line);
				// an absolute redirect to a lower page
				$line = preg_replace("#^Location: http://$knownhost/(.*)$#", "Location: $rewritebase/$knownhostAbbr/$1", $line);

				// Support subdomains

				// an absolute redirect to a main page
				$line = preg_replace("#^Location: http://(.*?)\.$knownhost#", "Location: $rewritebase/$1.$knownhostAbbr", $line);
				$line = preg_replace("#^Location: http://(.*?)\.$knownhost/#", "Location: $rewritebase/$1.$knownhostAbbr", $line);
				// an absolute redirect to a lower page
				$line = preg_replace("#^Location: http://(.*?)\.$knownhost/(.*)$#", "Location: $rewritebase/$1.$knownhostAbbr/$2", $line);

			}
		}

		// cookie handling
		if (preg_match("/^Set-Cookie: /", $line)) {

			if ($doCookies == false) {

				continue;

			}

			logline("Found cookie header $line");

			// rewrite cookie paths
			
			if (preg_match("#; path=([^;]*)#", $line, $matches)) {

				$pathNow = $matches[1];

				logline("Cookie path found $pathNow");

				if (preg_match("/^/", $pathNow)) {
	
					$pathNew = "$rewritebase/$abbreviation" . $pathNow;
					$line = preg_replace("#; path=([^;]*)#", "; path=$pathNew", $line);

					logline("Insecure path. Setting to $pathNew");
				}
			}

		}

		// get content type
		if (preg_match("/^Content-Type: text\/.*$/", $line)) {
			$text = true;
		}

		// ignore robots header. we have our own

		if (preg_match("/^X-Robots-Tag: /", $line)) {
			continue;
		}

		// pass through headers
	
		logline("Give header: $line");
		header($line);

	}

	// After a 30x relocation response, exit immediatly
	if ($redirect) {
		logline("EXIT because relocate");
		exit;
	}

	// add no robots header
	header("X-Robots-Tag: noindex, nofollow", true);

	if ($text) {

		/* Read text page and close handle */
		$contents = stream_get_contents($handle);
		fclose($handle);

//		logline("--BEGIN TEXT CONTENT--\n$contents\n--END CONTENT--");

	} else {
	
		/* Pass through binary content or non-parsed content */

		$size = fpassthru($handle);
		fclose($handle);
		register_bandwidth_usage($size);
		exit;

	}

	// Proxy simple, or use Proxyhelp

	if (!$proxyhelp) {

		// rewrite href root-relative urls
		$contents = preg_replace("#<(.*?)href=\"/([^/].*?)\"(.*?)>#", "<$1href=\"$rewritebase/$abbreviation/$2\"$3>", $contents);

		// rewrite form action root-relative urls
		$contents = preg_replace("#<form(.*?)action=\"/([^/].*?)\"(.*?)>#", "<form$1action=\"$rewritebase/$abbreviation/$2\"$3>", $contents);

		// rewrite src root-relative urls
		$contents = preg_replace("#<(.*?)src=\"/([^/].*?)\"(.*?)>#", "<$1src=\"$rewritebase/$abbreviation/$2\"$3>", $contents);

		// rewrite href absolute urls and form action absolute urls and src absolute urls
		foreach ($remoteservers as $host) {
			// future work: support hash matching in URLs instead of hostnames. entries for subdomains should be defined on the fly to make hashing possible
			$abbr = $host;

			$contents = preg_replace("#<(.*?)href=\"(http:)?//$host/(.*?)\"(.*?)>#", "<$1href=\"$rewritebase/$abbr/$3\"$4>", $contents);
			$contents = preg_replace("#<form(.*?)action=\"(http:)?//$host/(.*?)\"(.*?)>#", "<form$1action=\"$rewritebase/$abbr/$3\"$4>", $contents);
			$contents = preg_replace("#<(.*?)src=\"(http:)?//$host/(.*?)\"(.*?)>#", "<$1src=\"$rewritebase/$abbr/$3\"$4>", $contents);

			// support subdomains
			// future work: use preg_replace_all with callback to support hashing with subdomains

			$contents = preg_replace("#<(.*?)href=\"(http:)?//(.*?)\.$host/(.*?)\"(.*?)>#", "<$1href=\"$rewritebase/$3.$abbr/$4\"$5>", $contents);
			$contents = preg_replace("#<form(.*?)action=\"(http:)?//(.*?)\.$host/(.*?)\"(.*?)>#", "<form$1action=\"$rewritebase/$3.$abbr/$4\"$5>", $contents);
			$contents = preg_replace("#<(.*?)src=\"(http:)?//(.*?)\.$host/(.*?)\"(.*?)>#", "<$1src=\"$rewritebase/$3.$abbr/$4\"$5>", $contents);

		}

	} else {

		proxyhelp_init_for_repress($rewritebase, $abbreviation, $remoteservers);

		$contents = proxify_html_resource($contents);

	}

	/* Push text. */

	$size = strlen($contents);
	register_bandwidth_usage($size);
	echo $contents;

	/* Done. */

	exit;

}

function register_bandwidth_usage($size) {

	if ( function_exists('repress_register_bandwidth_usage') ) {
		repress_register_bandwidth_usage($size);
	}

}

/** Log and debug functions **/

function logline($line) {
	if (!$GLOBALS['proxydebug']) { return; }

	if (!is_link("/tmp/repress.module.debug.log")) {
		$f = fopen("/tmp/repress.module.debug.log", "a");
		fputs($f, $line . "\n");
		fclose($f);
	}
}

